#!/usr/bin/bash

# 列出已加载的模块
loaded_modules=$(lsmod | awk '{print $1}' | tail -n +2)

has_third_party_modules=false
third_party_modules=()

# 检查每个模块的来源
for module in $loaded_modules; do
    modinfo_output=$(modinfo "$module" 2>/dev/null)

    # 检查 modinfo 是否成功
    if [ $? -ne 0 ]; then
        third_party_modules+=("{\"module\": \"$module\", \"filename\": \"\", \"source\": \"Possibly temporarily installed\"}")
        has_third_party_modules=true
        continue
    fi

    filename=$(echo "$modinfo_output" | grep "^filename:" | awk '{print $2}')
    intree=$(echo "$modinfo_output" | grep "^intree:" | awk '{print $2}')

    if [ "$intree" != "Y" ]; then
        package=""
        if command -v dpkg &> /dev/null; then
            package=$(dpkg -S "$filename" 2>/dev/null)
        elif command -v rpm &> /dev/null; then
            package=$(rpm -qf "$filename" 2>/dev/null)
            if [[ "$package" == *"is not owned by any package"* ]]; then
                package=""
            fi
        fi

        if [ -z "$package" ]; then
            third_party_modules+=("{\"module\": \"$module\", \"filename\": \"$filename\", \"source\": \"Possibly user-installed\"}")
            has_third_party_modules=true
        fi
    fi
done

# 构建 JSON 输出
output="{\"third_party_modules\": [$(IFS=,; echo "${third_party_modules[*]}")]}"

# 使用 jq 格式化 JSON 输出
echo "$output" | jq .

# 根据是否有第三方模块设置退出状态
if [ "$has_third_party_modules" = true ]; then
    exit 1
else
    exit 0
fi
